import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.channels.Selector;
import java.nio.channels.SelectionKey;
import java.nio.ByteBuffer;
import java.io.IOException;
import java.util.Set;
import java.util.Iterator;
import java.net.InetSocketAddress;

public class NioIncreaserServer {

	public static void main(String[] args) throws IOException {
		int[] value = { 0 };

		Selector selector = Selector.open();

		System.out.println("Selector open: " + selector.isOpen());

		// Get server socket channel and register with selector
		ServerSocketChannel serverSocket = ServerSocketChannel.open();
		InetSocketAddress hostAddress = new InetSocketAddress("localhost", 12345);
		serverSocket.bind(hostAddress);
		serverSocket.configureBlocking(false);
		int ops = serverSocket.validOps();
		SelectionKey selectKy = serverSocket.register(selector, ops, null);

		for (;;) {
			System.out.println("Waiting for select...");
			int noOfKeys = selector.select();

			System.out.println( "Number of selected keys: " + noOfKeys);

			Set<SelectionKey> selectedKeys = selector.selectedKeys();
			Iterator<SelectionKey> iter = selectedKeys.iterator();

			while (iter.hasNext()) {
				SelectionKey ky = iter.next();

				if (ky.isAcceptable()) {
					// Accept the new client connection
					SocketChannel client = serverSocket.accept();
					client.configureBlocking(false);

					// Add the new connection to the selector
					client.register(selector, SelectionKey.OP_READ);

					System.out.println("Accepted new connection from client: " + client);
				}
				else if (ky.isReadable()) {
					SocketChannel client = (SocketChannel) ky.channel();
					ByteBuffer buffer = ByteBuffer.allocate(16);
					client.read(buffer);
					String input = new String(buffer.array()).trim();

					System.out.println("Message read from client: " + input);
	
					if (input.equals("Bye.")) {
						client.close();
						System.out.println("Client messages are complete; close.");
					}

					int inValue = Integer.parseInt(input);
					if (inValue <= value[0]) {
						ByteBuffer buf = ByteBuffer.wrap("NO\r\n".getBytes());
						buf.flip();
						client.write(buf);
					} else {
						value[0] = inValue;
						
						ByteBuffer buf = ByteBuffer.wrap("OK\r\n".getBytes());
						buf.flip();
						client.write(buf);
					}
					
				} // end if

				iter.remove();

			} // end while loop

		} // end for loop
	}
} // class

